% Code fragments for monadic structural function loop structures.
% 2004-08-14
%
%Fragment tran x00  x bidc bidc .
%Fragment rotr x00  x bidc bidc .
%Fragment rot1 x00  x bidc bidc .
inline $ZTYPE $FNAME($YTYPE y)
{ /* Tranpose and rotate on scalars are NOPs */ 
return(y);
}

%Fragment coma x01 x bidc bidc .
inline $ZTYPE[.] $FNAME($YTYPE y)
{ /* Ravel of scalar */
	return([y]);
}

%Fragment comb x02 x bidc bidc .
inline $ZTYPE[1,1] $FNAME($YTYPE y)
{ /* Table of scalar */
 z = reshape([1,1],y);
 return(z);
}

%Fragment comb x12 x bidc bidc .
inline $ZTYPE[.,1] $FNAME($YTYPE[.] y)
{ /* Table on vector */
 z = reshape([shape(y),1],y);
 return(z);
}

%Fragment comb x22 x bidc bidc .
inline $ZTYPE[.,.] $FNAME($YTYPE[.,.] y)
{ /* Table on table is NOP */
	return(y);
}

%Fragment comb x** x bidc bidc .
inline $ZTYPE[.,.] $FNAME($YTYPE[+] y)
{ /* Table on matrix */
 shpy = shape(y);
 z = reshape(take([1],shpy),prod(drop([1],shpy)),y);
 return(z);
}

%Fragment tran x11 x bidc bidc .
inline $ZTYPE[.] $FNAME($YTYPE[.] y)
{ /* Transpose on vectors is NOP */ 
	return(y);
}

%Fragment coma x11 x bidc bidc .
inline $ZTYPE[.] $FNAME($YTYPE[.] y)
{ /* Ravel of vector is NOP */  
	return(y);
}


%Fragment coma x*1 x bidc bidc .
inline $ZTYPE[.] $FNAME($YTYPE[+] y)
{ /* Ravel of anything with rank>1 */
	z = reshape([prod(shape(y))],y);
	return(z);
}

%Fragment rotr x11 x bidc bidc .
inline $ZTYPE[.] $FNAME($YTYPE[.] y)
{ /* Vector reverse */
 n = shape(y);
 cell = $OTFILL;
 z = with {
	( . <= iv <= .)
		: y[(n-1)-iv];
	} : genarray(n, cell);
 return(z);
}

%Fragment rot1 x** x bidc bidc .
inline $ZTYPE[+] $FNAME($YTYPE[+] y)
{ /* First axis reverse on anything */
 frameshape = take([1],shape(y));
 cell = genarray(drop([1],shape(y)), $OTFILL);
 z = with {
	( . <= iv <= .)
		: y[(frameshape-[1])-iv];
	} : genarray(frameshape, cell);
 return(z);
}

%Fragment rotr x** x bidc bidc .
inline $ZTYPE[+] $FNAME($YTYPE[+] y)
{/* Last axis reverse on rank>1 */
 cellshape = take([-1], shape(y));
 frameshape = drop([-1],shape(y));
 cell = genarray(cellshape, $OTFILL);
 z = with {
	( . <= iv <= .)
		: $FNAME(y[iv]);
	} : genarray(frameshape, cell);
 return(z);
}
%Generate , $FN, $XT$YT$ZT, x11, ., $CT

%Fragment tran x22 x bidc bidc .
inline $ZTYPE[.,.] $FNAME($YTYPE[.,.] y)
{ /* Transpose on rank-2 */
	z = { [i,j] -> y[j,i] };
	return(z);
}

%Fragment tran x** x bidc bidc .
inline $ZTYPE[+] $FNAME($YTYPE[+] y)
{ /* General transpose */
	z = TRANSPOSE(y);
	return(z);
}
%Generate ,   TRANSPOSE, X$YT$YT, X**, ., $YT

%
% Copyright (C) 2007 Robert Bernecky
%   This file is part of the APEX APL compiler run-time system.
%
%   The APEX APL compiler run-time system is free software; 
%   you can redistribute it and/or
%   modify it under the terms of the GNU Lesser General Public
%   License as published by the Free Software Foundation; either
%   version 2.1 of the License, or (at your option) any later version.
%
%   The APEX APL compiler run-time system is distributed in the 
%   hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%   Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public
%   License along with the GNU C Library; if not, write to the Free
%   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
%   02111-1307 USA.  
%
