% Code fragments for reshape fragments
%
% Rewritten for sac with help from Sven-Bodo Scholz.
% Robert Bernecky 2004-09-17. Add SAC-like rank specifiers

%Fragment rho 001 bid bidc bidc .
inline $ZTYPE[.] $FNAME($XTYPE x, $YTYPE y)
{ /* Scalar reshape scalar to vector) */ 
	z = genarray([toi(x)], y);
	return(z);
}

%Fragment rho 100 bidc bidc bidc .
inline $ZTYPE $FNAME($XTYPE[.] x, $YTYPE y)
{ /* Empty Vector reshape scalar to scalar */ 
 return(y);
}

%Fragment rho 10* bidc bidc bidc .
inline $ZTYPE[*] $FNAME($XTYPE[.] x, $YTYPE y)
{ /* Vector reshape scalar to matrix) */ 
	zxrho = prod(toi(x)); /* Result element count */
	z = genarray([zxrho], y); /* allocate result */
	z = reshape(toi(x),z);
	return(z);
}

%Fragment rho 0*1 bid bidc bidc .
inline $ZTYPE[.] $FNAME($XTYPE x, $YTYPE[+] y)
{ /* Scalar reshape non-scalar (to vector) */ 
 z = rhoI$YT$ZT([toi(x)],y);
 return(z);
}
%Generate , rho, I$YT$YT, 1*1, ., $YT

%Fragment rho 1** c bidc bidc .
inline $ZTYPE[*] $FNAME($XTYPE[.] x, $YTYPE[*] y)
{ /* Character-vector reshape anything. */
  /* Left argument must be empty vector! */
 if ( 0 != shape(x)[[0]]) {
	show(tochar("reshape domain error in $FNAME"));
 }
 z = reshape([prod(shape(y))], y);
 return(z[[0]]);
}

%Fragment rho 1** bid bidc bidc .
inline $ZTYPE[*] $FNAME($XTYPE[.] x, $YTYPE[+] y)
{ /* APEX vector x reshape, with item reuse */ 
  ix = toi(x);
  ry = comaX$YT$YT(y); 
  zxrho = prod(ix); /* THIS NEEDS XRHO FOR CODE SAFETY!! */ 
  yxrho = shape(ry)[[0]];
  if( zxrho <= yxrho) { /* No element resuse case */
        z = take([zxrho],ry); 
 } else { 
        ncopies = zxrho/yxrho; /* # complete copies of y. */
        /* FIXME: y empty case !*/
        z = with {
		(. <= [i] <= .)
			: y;
		} : genarray( [ncopies], y);
        /* Now append the leftover bits */
        z = comaX$YT$YT(z) ++ take([zxrho-(ncopies*yxrho)],ry); 
 } 
 return(reshape(ix,z)); 
}
%Generate , coma, X$YT$YT, X*1, ., $YT


%Fragment rho *** bidc bidc bidc .
inline $ZTYPE[*] $FNAME($XTYPE[*] x, $YTYPE[*] y)
{ /* x rank>1! rank error! */
 APEXERROR("rank error"++__FILE__++__LINE__);
 return(y);
}
%
% Copyright (C) 2007 Robert Bernecky
%   This file is part of the APEX APL compiler run-time system.
%
%   The APEX APL compiler run-time system is free software; 
%   you can redistribute it and/or
%   modify it under the terms of the GNU Lesser General Public
%   License as published by the Free Software Foundation; either
%   version 2.1 of the License, or (at your option) any later version.
%
%   The APEX APL compiler run-time system is distributed in the 
%   hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
%   Lesser General Public License for more details.
%
%   You should have received a copy of the GNU Lesser General Public
%   License along with the GNU C Library; if not, write to the Free
%   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
%   02111-1307 USA.  
%
